/*****************************************************************

	ActiveUI 0.2.2 - cross-browser DHTML widgets toolkit.
	Copyright (C) 2003 ActiveUI Software Ltd. All Rights Reserved. 
	More information at http://www.activeui.net/

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*****************************************************************/

Active.System.Template = Active.System.HTML.subclass();

Active.System.Template.create = function(){

/****************************************************************

	Generic HTML template class. Provides methods for retrieving
	values from data models and calling nested templates.

*****************************************************************/

	var obj = this.prototype;
	var _super = this.superclass.prototype;
	var _pattern = /^(\w+)\W(.+)$/;

	var join = function(){
		var i, s = arguments[0];
		for (i=1; i<arguments.length; i++){s += arguments[i].substr(0,1).toUpperCase() + arguments[i].substr(1)}
		return s;
	};

/****************************************************************

	Creates necessary methods for a new data model.

	@param	name	(String) New data model name.

*****************************************************************/

	obj.defineModel = function(name){

		var external = "_" + name + "Model";

		var defineProperty = join("define", name, "property");
		var definePropertyArray = join("define", name, "property", "array");
		var getProperty = join("get", name, "property");
		var setProperty = join("set", name, "property");
		var get = {};
		var set = {};

		var getModel = join("get", name, "model");
		var setModel = join("set", name, "model");
		var updateModel = join("update", name, "model");


//		------------------------------------------------------------

		this[defineProperty] = function(property, defaultValue){

			var _getProperty = join("get", name, property);
			var _setProperty = join("set", name, property);
			var _property = "_" + join(name, property);

			var getPropertyMethod = function(){
				return this[_property];
			};

			this[_getProperty] = getPropertyMethod;

			this[_setProperty] = function(value){
				if(typeof value == "function"){
					this[_getProperty] = value;
				}
				else {
					if (this[_getProperty] !== getPropertyMethod) {this[_getProperty] = getPropertyMethod}
					this[_property] = value;
				}
				this[updateModel](property);
			};

			this[_setProperty](defaultValue);
		};

//		------------------------------------------------------------

		this[getProperty] = function(property, a, b, c){
			try {
				if (this[external]) {return this[external].getProperty(property, a, b, c)}
				if (!get[property]) {get[property] = join("get", name, property)}
				return this[get[property]](a, b, c);
			}
			catch(error){
				return this.handle(error);
			}
		};

//		------------------------------------------------------------

		this[setProperty] = function(property, value, a, b, c){
			try {
				if (this[external]) {return this[external].setProperty(property, a, b, c)}
				if (!set[property]) {set[property] = join("set", name, property)}
				return this[set[property]](value, a, b, c);
			}
			catch(error){
				return this.handle(error);
			}
		};

//		------------------------------------------------------------

		obj[getProperty] = function(property, a, b, c){
			if (this[external]) {return this[external].getProperty(property, a, b, c)}
			return this.$owner[getProperty](property, a, b, c);
		};

		obj[setProperty] = function(property, value, a, b, c){
			if (this[external]) {return this[external].setProperty(property, a, b, c)}
			return this.$owner[setProperty](property, value, a, b, c);
		};

//		------------------------------------------------------------

		this[definePropertyArray] = function(property, defaultValue){

			var _getProperty = join("get", name, property);
			var _setProperty = join("set", name, property);
			var _getArray = join("get", name, property + "s");
			var _setArray = join("set", name, property + "s");
			var _array = "_" + join(name, property + "s");
			var _getCount = join("get", name, "count");
			var _setCount = join("set", name, "count");

			var getArrayElement = function(index){
				return this[_array][index];
			};

			var getStaticElement = function(){
				return this[_array];
			};

			var getArray = function(){
				return this[_array].concat();
			};

			var getTempArray = function(){
				var i, a = [], max = this[_getCount]();
				for(i=0; i<max; i++) {a[i] = this[_getProperty](i)}
				return a;
			};

			this[_setProperty] = function(value, index){
				if(typeof value == "function"){
					this[_getProperty] = value; 
					this[_getArray] = getTempArray;
				}
				else if (arguments.length==1) {
					this[_array] = value;
					this[_getProperty] = getStaticElement;
					this[_getArray] = getTempArray;
				}
				else {
					if (this[_getArray] != getArray) {this[_array] = this[_getArray]()}
					this[_array][index] = value;
					this[_getProperty] = getArrayElement;
					this[_getArray] = getArray;
				}
				this[updateModel](property);
			};

			this[_setArray] = function(value){
				if(typeof value == "function"){
					this[_getArray] = value;
				}
				else {
					this[_array] = value.concat();
					this[_getProperty] = getArrayElement;
					this[_getArray] = getArray;
					this[_setCount](value.length);
				}
				this[updateModel](property);
			};

			this[_setProperty](defaultValue);
		};

//		------------------------------------------------------------

		var proxyPrototype = new Active.System.Model;

		proxyPrototype.getProperty = function(property, a, b, c){
			return this._target[getProperty](property, a, b, c);
		};

		var proxy = join("_", name, "proxy");

		this[getModel] = function(){
			if (this[external]) {return this[external]}
			if (!this[proxy]) {
				this[proxy] = proxyPrototype.clone();
				this[proxy]._target = this;
				this[proxy].$owner = this.$owner; 
			}
			return this[proxy];
		};

		obj[setModel] = function(model){
			this[external] = model;
			if (model && !model.$owner) {model.$owner = this}
		};

		obj[getModel] = function(a, b, c){
			if (this[external]) {return this[external]}
			return this.$owner[getModel](a, b, c);
		};

//		------------------------------------------------------------

		this[updateModel] = function(){};

	};

/****************************************************************

	Returns an instance of the data model with a given name,
	defined on the template.

	@param	name	(String) Name of the data model.
	@return			A data model object.

*****************************************************************/

	obj.getModel = function(name){
		var getModel = join("get", name, "model");
		return this[getModel]();
	};

	obj.setModel = function(name, model){
		var setModel = join("set", name, "model");
		return this[setModel](model);
	};

//	------------------------------------------------------------

	obj.defineProperty = function(name, defaultValue){
		if (name.match(_pattern)) {
			var defineProperty = join("define", RegExp.$1, "property");
			if (this[defineProperty]) {return this[defineProperty](RegExp.$2, defaultValue)}
		}
	};

	obj.definePropertyArray = function(name, defaultValue){
		if (name.match(_pattern)) {
			var defineArray = join("define", RegExp.$1, "property", "array");
			if (this[defineArray]) {return this[defineArray](RegExp.$2, defaultValue)}
		}
	};

	obj.getProperty = function(name, a, b, c){
		if (name.match(_pattern)) {
			var getProperty = join("get", RegExp.$1, "property");
			if (this[getProperty]) {return this[getProperty](RegExp.$2, a, b, c)}
		}
	};

	obj.setProperty = function(name, value, a, b, c){
		if (name.match(_pattern)) {
			var setProperty = join("set", RegExp.$1, "property");
			if (this[setProperty]) {return this[setProperty](RegExp.$2, value, a, b, c)}
		}
	};

//	------------------------------------------------------------

	obj.defineTemplate = function(name, template){

		var ref = "_" + name + "Template";
		var get = join("get", name, "template");
		var set = join("set", name, "template");
		var getDefault = join("default", name, "template");

		var name1 = "." + name;
		var name2 = "." + name + ":";

		this[get] = this[getDefault] = function(index){
			if (this[ref].$owner != this) {this[set](this[ref].clone())}
			if (typeof(index)=="undefined") {
				this[ref]._id = this._id + name1;
			}
			else {
				this[ref]._id = this._id + name2 + index;
			}
			this[ref].$index = index;
			return this[ref];
		};

		obj[get] = function(a, b, c){
			return this.$owner[get](a, b, c);
		};

		obj[set] = function(template){
			this[ref] = template;
			if (template) {
				template.$owner = this; 
			}
		};

		this[set](template);
	};


//	------------------------------------------------------------


	obj.getTemplate = function(name){
		if (name.match(_pattern)) {
			var get = join("get", RegExp.$1, "template");
			arguments[0] = RegExp.$2;
			var template = this[get]();
			return template.getTemplate.apply(template, arguments);
		}
		else {
			get = join("get", name, "template");
			var i, args = [];
			for(i=1; i<arguments.length; i++) {args[i-1]=arguments[i]}
			return this[get].apply(this, args);
		}
	};

//	------------------------------------------------------------

	obj.setTemplate = function(name, template, index){
		if (name.match(_pattern)) {
			var get = join("get", RegExp.$1, "template");
			var n = RegExp.$2;
			this[get]().setTemplate(n, template, index);
		}
		else {
			var set = join("set", name, "template");
			this[set](template, index);
		}
	};

//	------------------------------------------------------------

	obj.setAction = function(name, value){
		this["_" + name + "Action"] = value;
	};

	obj.action = function(name, source){
		if (!source) {source = this}
		var action = this["_" + name + "Action"];
		if (typeof(action)=="function") {action.call(this, source)}
		else if (this.$owner) {this.$owner.action(name, source)}
	};

//	------------------------------------------------------------

};

Active.System.Template.create();